@@ -1,33 +1,51 @@
 package WebService::EveOnline::API::Character;
 
-=head2 $eve->characters
+use base qw/ WebService::EveOnline::Base /;
 
-Returns an array of character objects for each characters available via
-your API key.
+our $VERSION = "0.60";
+
+=head4 new
+
+This is called under the hood when an $eve->character object is requested.
+
+It returns an array of character objects for each characters available via
+your API key. You probably won't need to call this directly.
 
 =cut
 
-sub characters {
-	my ($self, $params) = @_;
-	my $character_data = $self->call_api('character', {});
-	my $characters = [];
-	foreach my $character (sort keys %{$character_data}) {
-		next if $character =~ /^_/; # skip meta keys
-		push(@{$characters}, bless({ 
+sub new {
+    my ($self, $c, $name) = @_;
+
+    my $character_data = $self->call_api('character', {}, $c);
+
+    # XML::Simple is a big pile of shit:
+    $character_data = ($character_data->{name}) ? { $character_data->{name} => $character_data } : $character_data;
+
+    my $characters = [];
+    foreach my $character (sort keys %{$character_data}) {
+        next if $character =~ /^_/; # skip meta keys
+        
+        my $char_obj = bless({ 
                  _character_name => $character, 
                  _corporation_name => $character_data->{$character}->{corporationName},
                  _corporation_id => $character_data->{$character}->{corporationID},
                  _character_id => $character_data->{$character}->{characterID},
-                 _api_key => $self->api_key,
-                 _user_id => $self->user_id,
-                 _evecache => $self->{_evecache},
-            }, ref($self))
-		);
-	}
-	return @{$characters};
+                 _api_key => $c->{_api_key},
+                 _user_id => $c->{_user_id},
+                 _evecache => $c->{_evecache},
+            }, __PACKAGE__ );
+        
+        if ($name) {
+            return $char_obj if $char_obj->name eq $name;
+        } else {
+            push(@{$characters}, $char_obj);
+        }
+    }
+    
+    return @{$characters};
 }
 
-=head2 $character->character_hashref
+=head2 $character->hashref
 
 Returns a character hashref on a character object containing the following keys:
 
@@ -41,7 +59,7 @@ Returns a character hashref on a character object containing the following keys:
 
 =cut
 
-sub character_hashref {
+sub hashref {
     my ($self) = @_;
     return {
         character_name => $self->{_character_name},
@@ -54,77 +72,105 @@ sub character_hashref {
     };
 }
 
-=head2 $eve->character($id || $name)
+=head2 assets
 
-Returns a character object given a particular id or name.
+Placeholder
 
 =cut
 
-sub character {
-	my ($self, $id) = @_;
-    foreach my $character ($self->characters) {
-        return $character if $character->character_name eq $id || $character->character_id == $id; 
-    }
-	return undef;
+sub assets {
+    my ($self, $params) = @_;
+    my $assets = $self->call_api('assets', { characterID => $self->{_character_id} }, $self);
+    return $assets;   
 }
 
-=head2 $character->character_name
+=head2 kills
+
+Placeholder
+
+=cut
+
+sub kills {
+    my ($self, $params) = @_;
+    my $kills = $self->call_api('kills', { characterID => $self->{_character_id} }, $self);
+    return $kills;   
+}
+
+=head2 orders
+
+Placeholder
+
+=cut
+
+sub orders {
+    my ($self, $params) = @_;
+    my $orders = $self->call_api('orders', { characterID => $self->{_character_id} }, $self);
+    return $orders;   
+}
+
+=head2 $character->name
 
 Returns the name of the current character based on the character object.
 
 =cut
 
-sub character_name {
-	my ($self) = @_;
+sub name {
+    my ($self) = @_;
     return $self->{_character_name};
 }
 
-=head2 $character->character_id
+=head2 $character->id
 
 Returns a character object based on the character id you provide, assuming
 your API key allows it.
 
 =cut
 
-sub character_id {
-	my ($self) = @_;
-	return $self->{_character_id};		
+sub id {
+    my ($self) = @_;
+    return $self->{_character_id};      
 }
 
-=head2 $character->character_race
+=head2 $character->race
 
 The race of the selected character.
 
 =cut
 
-sub character_race {
-	my ($self, $params) = @_;
-	my $race = $self->call_api('race', { characterID => $self->character_id });
-	return $race->{race};	
+sub race {
+    my ($self, $params) = @_;
+    my $race = $self->call_api('race', { characterID => $self->{_character_id} }, $self);
+    return $race->{race};   
 }
 
-=head2 $character->character_bloodline
+=head2 $character->bloodline
 
 The bloodline of the selected character.
 
 =cut
 
-sub character_bloodline {
-	my ($self, $params) = @_;
-	my $bloodline = $self->call_api('bloodline', { characterID => $self->character_id });
-	return $bloodline->{bloodLine};	
+sub bloodline {
+    my ($self, $params) = @_;
+    my $bloodline = $self->call_api('bloodline', { characterID => $self->{_character_id} }, $self);
+    return $bloodline->{bloodLine}; 
 }
 
-=head2 $character->character_gender
+=head2 $character->gender, sex
 
 The gender of the selected character.
 
 =cut
 
-sub character_gender {
-	my ($self, $params) = @_;
-	my $gender = $self->call_api('gender', { characterID => $self->character_id });
-	return $gender->{gender};	
+sub gender {
+    my ($self, $params) = @_;
+    my $gender = $self->call_api('gender', { characterID => $self->{_character_id} }, $self);
+    return $gender->{gender};   
+}
+
+sub sex {
+    my ($self, $params) = @_;
+    my $gender = $self->call_api('gender', { characterID => $self->{_character_id} }, $self);
+    return $gender->{gender};   
 }
 
 =head2 $character->attributes
@@ -134,8 +180,8 @@ Sets the base attributes held by the selected character.
 =cut
 
 sub attributes {
-	my ($self, $params) = @_;
-	my $attributes = $self->call_api('attributes', { characterID => $self->character_id });
+    my ($self, $params) = @_;
+    my $attributes = $self->call_api('attributes', { characterID => $self->{_character_id} }, $self);
 
     $self->{_attributes} = {
         _memory => $attributes->{memory},
@@ -145,7 +191,7 @@ sub attributes {
         _willpower => $attributes->{willpower},
     };
 
-	return bless($self, ref($self));	
+    return bless($self, __PACKAGE__);    
 }
 
 =head2 $character->attributes->memory, $attributes->memory
@@ -236,21 +282,9 @@ interface, so use with caution.
 =cut
 
 sub attribute_enhancers {
-	my ($self, $params) = @_;
-	my $enhancers = $self->call_api('enhancers', { characterID => $self->character_id });
-	return $enhancers;	
-}
-
-=head2 $character->account_balance
-
-The account balance of the selected character.
-
-=cut
-
-sub account_balance {
-	my ($self, $params) = @_;
-	my $balance = $self->call_api('balance', { characterID => $self->character_id });
-	return $balance->{balance};
+    my ($self, $params) = @_;
+    my $enhancers = $self->call_api('enhancers', { characterID => $self->{_character_id} }, $self);
+    return $enhancers;  
 }
 
 1;
